@@ -22,29 +22,61 @@ import os
 import re
 import shutil
 import glob
+import datetime
 import numpy as np
 import pandas as pd
+import xarray as xr
 import dask.dataframe as dd
+import importlib.metadata
+from typing import Union
 from disdrodb.utils.logger import log_info, log_warning
+from disdrodb.L0.metadata import read_metadata
+
 logger = logging.getLogger(__name__)
 
 ####---------------------------------------------------------------------------.
 #### Directory/Filepaths Defaults
-def infer_institute_from_fpath(fpath):
-    idx_start = fpath.rfind("DISDRODB")
-    disdrodb_fpath = fpath[idx_start:]
-    institute = disdrodb_fpath.split("/")[2]   
-    return institute 
+def infer_institute_from_fpath(fpath: str) -> str:
+    """Infer institue name from file path.
 
-def infer_campaign_from_fpath(fpath):
-    idx_start = fpath.rfind("DISDRODB")
-    disdrodb_fpath = fpath[idx_start:]
-    campaign = disdrodb_fpath.split("/")[3] 
-    return campaign 
+    Parameters
+    ----------
+    fpath : str
+        Input file path.
 
-def infer_station_id_from_fpath(fpath):
+    Returns
+    -------
+    str
+        Name of the institute.
     """
-    Get the station ID from the path of the input raw data. 
+    idx_start = fpath.rfind("DISDRODB")
+    disdrodb_fpath = fpath[idx_start:]
+    institute = disdrodb_fpath.split("/")[2]
+    return institute
+
+
+def infer_campaign_from_fpath(fpath: str) -> str:
+    """Infer campaign name from file path.
+
+    Parameters
+    ----------
+    fpath : str
+        Input file path.
+
+    Returns
+    -------
+    str
+        Name of the campaign.
+    """
+    idx_start = fpath.rfind("DISDRODB")
+    disdrodb_fpath = fpath[idx_start:]
+    campaign = disdrodb_fpath.split("/")[3]
+    return campaign
+
+
+def infer_station_id_from_fpath(fpath: str) -> str:
+    """
+    Get the station ID from the path of the input raw data.
 
     Parameters
     ----------
@@ -61,59 +93,251 @@ def infer_station_id_from_fpath(fpath):
     disdrodb_fpath = fpath[idx_start:]
     list_path_elements = re.split(path_pattern, disdrodb_fpath)
     station_id = list_path_elements[10]
-    # Optional strip .yml if fpath point to YAML file 
-    station_id.strip(".yml")  
-    return station_id 
+    # Optional strip .yml if fpath point to YAML file
+    station_id.strip(".yml")
+    return station_id
 
 
-def get_campaign_name(base_dir):
-    """Return the campaign name from 'raw_dir' or processed_dir' paths."""
+def get_campaign_name(base_dir: str) -> str:
+    """Return the campaign name from 'raw_dir' or 'processed_dir' paths.
+
+    Parameters
+    ----------
+    base_dir : str
+        Path 'raw_dir' or 'processed_dir' directory.
+
+    Returns
+    -------
+    str
+        Name of the campaign.
+    """
+
     base_dir = parse_fpath(base_dir)
     campaign_name = os.path.basename(base_dir).upper()
     return campaign_name
 
-def get_L0A_dir(processed_dir, station_id): 
+
+def get_data_source(base_dir: str) -> str:
+    """Retrieves the data source from 'raw_dir' or processed_dir' paths
+
+    Parameters
+    ----------
+    base_dir : str
+        Input paths
+
+    Returns
+    -------
+    str
+        Name of the data source
+    """
+
+    base_dir = parse_fpath(base_dir)
+    institute_name = os.path.basename(os.path.dirname(base_dir)).upper()
+    return institute_name
+
+
+def get_dataset_min_max_time(ds: xr.Dataset):
+    """Retrieves dataset starting and ending time.
+
+    Parameters
+    ----------
+    ds : xr.Dataset
+        Input dataset
+
+    Returns
+    -------
+    tuple
+        (starting_time, ending_time)
+
+    """
+
+    starting_time = ds["time"].values[0]
+    ending_time = ds["time"].values[-1]
+    return (starting_time, ending_time)
+
+
+# TODO: get_dataframe_min_max_time
+
+
+def get_L0A_dir(processed_dir: str, station_id: str) -> str:
+    """Get L0A directory
+
+    Parameters
+    ----------
+    processed_dir : str
+        Path of the processed directory
+    station_id : int
+        ID of the station
+
+    Returns
+    -------
+    str
+        L0A directory path.
+    """
     dir_path = os.path.join(processed_dir, "L0A", station_id)
     return dir_path
 
-def get_L0A_fname(campaign_name, station_id, suffix=""):
+
+def get_L0A_fname(campaign_name: str, station_id: str, suffix: str = "") -> str:
+    """build L0A file name.
+
+    Parameters
+    ----------
+    campaign_name : str
+        Name of the campaign.
+    station_id : int
+        ID of the station
+    suffix : int, optional
+        suffix, by default ""
+
+    Returns
+    -------
+    str
+        L0A file name.
+    """
     if suffix != "":
         suffix = "_" + suffix
     fname = campaign_name + "_s" + station_id + suffix + ".parquet"
     return fname
 
 
-def get_L0A_fpath(processed_dir, station_id, suffix=""):
+# TODO: and refactor L0_processing --> remove suffix
+#
+# def get_L0A_fname(df, processed_dir, station_id: str) -> str:
+#     """Define L0A file name.
+
+#     Parameters
+#     ----------
+#     ds : pd.DataFrame
+#         L0A DataFrame
+#     processed_dir : str
+#         Path of the processed directory
+#     station_id : int
+#         ID of the station
+
+#     Returns
+#     -------
+#     str
+#         L0B file name.
+#     """
+#     starting_time, ending_time = get_dataframe_min_max_time(ds)
+#     starting_time = pd.to_datetime(starting_time).strftime("%Y%m%d%H%M%S")
+#     ending_time = pd.to_datetime(ending_time).strftime("%Y%m%d%H%M%S")
+#     # production_time = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
+#     campaign_name = get_campaign_name(processed_dir).replace(".", "-")
+#     institute_name = get_data_source(processed_dir).replace(".", "-")
+#     metadata_dict = read_metadata(processed_dir, station_id)
+#     sensor_name = metadata_dict.get("sensor_name").replace("_", "-")
+#     version = importlib.metadata.version("disdrodb").replace(".", "-")
+#     if version == "-VERSION-PLACEHOLDER-":
+#         version = "dev"
+#     fname = f"DISDRODB.L0A.Raw.{institute_name}.{campaign_name}.{station_id}.{sensor_name}.s{starting_time}.e{ending_time}.{version}.parquet"
+#     return fname
+
+
+def get_L0A_fpath(processed_dir: str, station_id: str, suffix: str = "") -> str:
+    """build L0A file path.
+
+    Parameters
+    ----------
+    campaign_name : str
+        Name of the campaign.
+    station_id : int
+        ID of the station
+    suffix : int, optional
+        suffix, by default ""
+
+    Returns
+    -------
+    str
+        L0A file path.
+    """
     campaign_name = get_campaign_name(processed_dir)
     fname = get_L0A_fname(campaign_name, station_id, suffix=suffix)
     dir_path = get_L0A_dir(processed_dir, station_id)
     fpath = os.path.join(dir_path, fname)
     return fpath
 
-def get_L0B_dir(processed_dir, station_id): 
+
+def get_L0B_dir(processed_dir: str, station_id: str) -> str:
+    """Build L0B directory
+
+    Parameters
+    ----------
+    processed_dir : str
+        Path of the processed directory
+    station_id : int
+        ID of the station
+
+    Returns
+    -------
+    str
+        Path of the L0B directory
+    """
     dir_path = os.path.join(processed_dir, "L0B", station_id)
     return dir_path
 
-def get_L0B_fname(campaign_name, station_id, suffix=""):
-    if suffix != "":
-        suffix = "_" + suffix
-    # TODO: _s make sense with station_id... but if station_name a bit orrible
-    fname = campaign_name + "_s" + station_id + suffix + ".nc"
+
+def get_L0B_fname(ds, processed_dir, station_id: str) -> str:
+    """Define L0B file name.
+
+    Parameters
+    ----------
+    ds : xr.Dataset
+        L0B xarray Dataset
+    processed_dir : str
+        Path of the processed directory
+    station_id : int
+        ID of the station
+
+    Returns
+    -------
+    str
+        L0B file name.
+    """
+    starting_time, ending_time = get_dataset_min_max_time(ds)
+    starting_time = pd.to_datetime(starting_time).strftime("%Y%m%d%H%M%S")
+    ending_time = pd.to_datetime(ending_time).strftime("%Y%m%d%H%M%S")
+    # production_time = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
+    # institute_name = get_data_source(processed_dir).replace(".", "-") # TODO: data_source
+    campaign_name = get_campaign_name(processed_dir).replace(".", "-")
+    metadata_dict = read_metadata(processed_dir, station_id)
+    sensor_name = metadata_dict.get("sensor_name").replace("_", "-")
+    version = importlib.metadata.version("disdrodb").replace(".", "-")
+    if version == "-VERSION-PLACEHOLDER-":
+        version = "dev"
+    fname = f"DISDRODB.L0B.Raw.{campaign_name}.{station_id}.{sensor_name}.s{starting_time}.e{ending_time}.{version}.nc"
     return fname
 
 
-def get_L0B_fpath(processed_dir, station_id, suffix=""):
-    campaign_name = get_campaign_name(processed_dir)
-    fname = get_L0B_fname(campaign_name, station_id, suffix=suffix)
+def get_L0B_fpath(ds, processed_dir: str, station_id: str) -> str:
+    """Define L0B file path.
+
+    Parameters
+    ----------
+    ds : xr.Dataset
+        L0B xarray Dataset
+    processed_dir : str
+        Path of the processed directory
+    station_id : int
+        ID of the station
+
+    Returns
+    -------
+    str
+        L0B file path.
+    """
     dir_path = get_L0B_dir(processed_dir, station_id)
+    fname = get_L0B_fname(ds, processed_dir, station_id)
     fpath = os.path.join(dir_path, fname)
     return fpath
 
+
 ####--------------------------------------------------------------------------.
 #### Directory/File Creation/Deletion
 
 
-def _create_directory(path):
+def _create_directory(path: str) -> None:
     if not isinstance(path, str):
         raise TypeError("'path' must be a strig.")
     try:
@@ -129,7 +353,7 @@ def _create_directory(path):
         raise FileNotFoundError(msg)
 
 
-def _remove_if_exists(fpath, force=False):
+def _remove_if_exists(fpath: str, force: bool = False) -> None:
     if os.path.exists(fpath):
         if not force:
             msg = f"--force is False and a file already exists at:{fpath}"
@@ -159,8 +383,25 @@ def _remove_if_exists(fpath, force=False):
 
 ####--------------------------------------------------------------------------.
 #### Directory checks
-def parse_fpath(fpath):
-    """Ensure fpath does not end with /."""
+def parse_fpath(fpath: str) -> str:
+    """Ensure fpath does not end with /.
+
+    Parameters
+    ----------
+    fpath : str
+        Input file path
+
+    Returns
+    -------
+    str
+        Output file path
+
+    Raises
+    ------
+    TypeError
+        Error il file path not compliant
+    """
+
     if not isinstance(fpath, str):
         raise TypeError("'parse_fpath' expects a directory/filepath string.")
     if fpath[-1] == "/":
@@ -168,9 +409,10 @@ def parse_fpath(fpath):
         fpath = fpath[:-1]
     return fpath
 
+
 ####--------------------------------------------------------------------------.
 #### L0 processing directory checks
-def check_raw_dir(raw_dir, verbose=False):
+def check_raw_dir(raw_dir: str, verbose: bool = False) -> None:
     """Check validity of raw_dir.
 
     Steps:
@@ -179,11 +421,27 @@ def check_raw_dir(raw_dir, verbose=False):
     3. Check that each station_id directory contains data
     4. Check that for each station_id the mandatory metadata are specified.
 
+
+
+    Parameters
+    ----------
+    raw_dir : str
+        Input raw directory
+    verbose : bool, optional
+        Wheter to verbose the processing.
+        The default is False.
+
+    Raises
+    ------
+    TypeError
+        Error if not complient.
     """
+
     from disdrodb.L0.metadata import create_metadata
     from disdrodb.L0.metadata import check_metadata_compliance
     from disdrodb.L0.issue import create_issue_yml
     from disdrodb.L0.issue import check_issue_compliance
+
     # -------------------------------------------------------------------------.
     # Check input argument
     if not isinstance(raw_dir, str):
@@ -201,17 +459,17 @@ def check_raw_dir(raw_dir, verbose=False):
         raise ValueError(
             "'raw_dir' {} should have the /data subfolder.".format(raw_dir)
         )
-    
+
     # -------------------------------------------------------------------------.
     #### Check there are subfolders corresponding to station to process
-    raw_data_dir= os.path.join(raw_dir, "data")
+    raw_data_dir = os.path.join(raw_dir, "data")
     list_data_station_id = os.listdir(raw_data_dir)
     if len(list_data_station_id) == 0:
         raise ValueError("No station directories within {}".format(raw_data_dir))
-    
+
     # -------------------------------------------------------------------------.
     #### Check there are data files in each list_data_station_id
-    list_raw_data_station_dir= [
+    list_raw_data_station_dir = [
         os.path.join(raw_data_dir, station_id) for station_id in list_data_station_id
     ]
     list_nfiles_per_station = [
@@ -223,12 +481,12 @@ def check_raw_dir(raw_dir, verbose=False):
         raise ValueError(
             "The following data directories are empty: {}".format(empty_station_dir)
         )
-        
+
     # -------------------------------------------------------------------------.
     #### Check there is /metadata subfolders
     metadata_dir = os.path.join(raw_dir, "metadata")
     if "metadata" not in list_subfolders:
-        # - Create metadata directory 
+        # - Create metadata directory
         _create_directory(metadata_dir)
         # - Create default metadata yml file for each station (since the folder didn't existed)
         list_metadata_fpath = [
@@ -245,7 +503,7 @@ def check_raw_dir(raw_dir, verbose=False):
     list_metadata_fpath = glob.glob(os.path.join(metadata_dir, "*.yml"))
     list_metadata_fname = [os.path.basename(fpath) for fpath in list_metadata_fpath]
     list_metadata_station_id = [fname[:-4] for fname in list_metadata_fname]
-    
+
     # - Check there is metadata for each station
     missing_data_station_idx = np.where(
         np.isin(list_data_station_id, list_metadata_station_id, invert=True)
@@ -264,9 +522,11 @@ def check_raw_dir(raw_dir, verbose=False):
             list_missing_station_id
         )
         raise ValueError(msg + " Now have been created to be filled.")
-        
+
     # - Check not excess metadata compared to present stations
-    excess_metadata_station_idx = np.where(np.isin(list_metadata_station_id, list_data_station_id, invert=True))[0]
+    excess_metadata_station_idx = np.where(
+        np.isin(list_metadata_station_id, list_data_station_id, invert=True)
+    )[0]
     if len(excess_metadata_station_idx) > 0:
         list_excess_station_id = [
             list_metadata_station_id[idx] for idx in excess_metadata_station_idx
@@ -276,7 +536,7 @@ def check_raw_dir(raw_dir, verbose=False):
                 list_excess_station_id
             )
         )
-        
+
     # -------------------------------------------------------------------------.
     #### Check metadata compliance
     _ = [check_metadata_compliance(fpath) for fpath in list_metadata_fpath]
@@ -285,21 +545,22 @@ def check_raw_dir(raw_dir, verbose=False):
     #### Check there is /issue subfolder
     issue_dir = os.path.join(raw_dir, "issue")
     if "issue" not in list_subfolders:
-        # - Create issue directory 
+        # - Create issue directory
         _create_directory(issue_dir)
         # - Create issue yml file for each station (since the folder didn't existed)
-        list_issue_fpath = [os.path.join(issue_dir, station_id + ".yml")
+        list_issue_fpath = [
+            os.path.join(issue_dir, station_id + ".yml")
             for station_id in list_data_station_id
         ]
         _ = [create_issue_yml(fpath) for fpath in list_issue_fpath]
         msg = "The /issue subfolder has been now created to document and then remove timesteps with problematic data."
         logger.info(msg)
     # -------------------------------------------------------------------------.
-    #### Check there are issue file for each station_id in /issue 
+    #### Check there are issue file for each station_id in /issue
     list_issue_fpath = glob.glob(os.path.join(issue_dir, "*.yml"))
     list_issue_fname = [os.path.basename(fpath) for fpath in list_issue_fpath]
     list_issue_station_id = [fname[:-4] for fname in list_issue_fname]
-    
+
     # - Check there is issue for each station
     missing_data_station_idx = np.where(
         np.isin(list_data_station_id, list_issue_station_id, invert=True)
@@ -317,12 +578,16 @@ def check_raw_dir(raw_dir, verbose=False):
         msg = "The issue files for the following station_id were missing: {}".format(
             list_missing_station_id
         )
-        
+
     # - Check not excess issue compared to present stations
-    excess_issue_station_idx = np.where(np.isin(list_issue_station_id, list_data_station_id, invert=True))[0]
+    excess_issue_station_idx = np.where(
+        np.isin(list_issue_station_id, list_data_station_id, invert=True)
+    )[0]
     if len(excess_issue_station_idx) > 0:
-        list_excess_station_id = [list_issue_station_id[idx] for idx in excess_issue_station_idx]
-        msg =  f"There are the following issue files without corresponding data: {list_excess_station_id}"
+        list_excess_station_id = [
+            list_issue_station_id[idx] for idx in excess_issue_station_idx
+        ]
+        msg = f"There are the following issue files without corresponding data: {list_excess_station_id}"
         log_warning(logger, msg, verbose)
 
     # -------------------------------------------------------------------------.
@@ -330,59 +595,102 @@ def check_raw_dir(raw_dir, verbose=False):
     _ = [check_issue_compliance(fpath) for fpath in list_issue_fpath]
     # TODO: MISSING IMPLEMENTATION OF check_issue_compliance
     # -------------------------------------------------------------------------.
-    return None
 
-def check_processed_dir(processed_dir, force=False):
-    """Check that 'processed_dir' is a valid directory path."""
+
+def check_processed_dir(processed_dir: str, force: bool = False) -> None:
+    """Check that 'processed_dir' is a valid directory path.
+
+    Parameters
+    ----------
+    processed_dir : str
+        Path of the processed directory
+    force : bool, optional
+        If True, overwrite existing data into processed directory.
+        If False, raise an error if there are already data into processed directory.
+
+    Raises
+    ------
+    TypeError
+        Error if path pattern not respected or can not be created.
+
+
+    """
+
     if not isinstance(processed_dir, str):
         raise TypeError("Provide 'processed_dir' as a string'.")
-    #------------------------------    
-    # Check processed_dir has "DISDRODB/Processed" to avoid deleting precious stuffs 
-    if processed_dir.find("DISDRODB/Processed") == -1 and processed_dir.find("DISDRODB\\Processed") == -1 : 
+    # ------------------------------
+    # Check processed_dir has "DISDRODB/Processed" to avoid deleting precious stuffs
+    if (
+        processed_dir.find("DISDRODB/Processed") == -1
+        and processed_dir.find("DISDRODB\\Processed") == -1
+    ):
         msg = "Expecting 'processed_dir' to contain the pattern */DISDRODB/Processed/*. or *\DISDRODB\Processed\*."
         logger.error(msg)
         raise ValueError(msg)
-        
-    # Check processed_dir does not end with "DISDRODB/Processed" 
-    # - It must contain also the <campaign_name> directory  
-    if processed_dir.endswith("Processed") or processed_dir.endswith("Processed/") or processed_dir.endswith("Processed\\"):
+
+    # Check processed_dir does not end with "DISDRODB/Processed"
+    # - It must contain also the <campaign_name> directory
+    if (
+        processed_dir.endswith("Processed")
+        or processed_dir.endswith("Processed/")
+        or processed_dir.endswith("Processed\\")
+    ):
         msg = "Expecting 'processed_dir' to contain the pattern */DISDRODB/Processed/<campaign_name>."
         logger.error(msg)
         raise ValueError(msg)
 
-    #------------------------------
-    # If forcing overwriting 
-    if force: 
-        # Check processed_dir is a directory before removing the content 
+    # ------------------------------
+    # If forcing overwriting
+    if force:
+        # Check processed_dir is a directory before removing the content
         if os.path.exists(processed_dir):
             if not os.path.isdir(processed_dir):
                 msg = f"'processed_dir' {processed_dir} already exist but is not a directory."
                 logger.error(msg)
-                raise ValueError(msg) 
+                raise ValueError(msg)
             # Remove content of existing processed_dir
             # TODO: maybe remove also the campaign name directory, not only the content of it
             shutil.rmtree(processed_dir)  # TODO: !! TOO DANGEROUS ??? !!!
 
-    #------------------------------
-    # If avoiding overwriting 
+    # ------------------------------
+    # If avoiding overwriting
     if not force:
         if os.path.exists(processed_dir):
             msg = f"'processed_dir' {processed_dir} already exists and force=False."
             logger.error(msg)
             raise ValueError(msg)
-            
-    #------------------------------        
+
+    # ------------------------------
     # Recreate processed_dir
     if not os.path.exists(processed_dir):
         os.makedirs(processed_dir)
     else:
-        msg = "Please report the BUG. This should not happen."  
+        msg = "Please report the BUG. This should not happen."
         logger.error(msg)
         raise ValueError(msg)
-           
 
-def check_campaign_name(raw_dir, processed_dir):
-    """Check that 'raw_dir' and 'processed_dir' have same campaign_name."""
+
+def check_campaign_name(raw_dir: str, processed_dir: str) -> str:
+    """Check that 'raw_dir' and 'processed_dir' have same campaign_name.
+
+    Parameters
+    ----------
+    raw_dir : str
+        Path of the raw directory
+    processed_dir : str
+        Path of the processed directory
+
+    Returns
+    -------
+    str
+        Campaign name in capital letter
+
+    Raises
+    ------
+    ValueError
+        Error if both paths do not match.
+    """
+
     upper_campaign_name = os.path.basename(raw_dir).upper()
     raw_campaign_name = os.path.basename(raw_dir)
     processed_campaign_name = os.path.basename(processed_dir)
@@ -390,7 +698,7 @@ def check_campaign_name(raw_dir, processed_dir):
         msg = f"'raw_dir' and 'processed_dir' must ends with same <campaign_name> {upper_campaign_name}"
         logger.error(msg)
         raise ValueError(msg)
-            
+
     if raw_campaign_name != upper_campaign_name:
         msg = f"'raw_dir' and 'processed_dir' must ends with UPPERCASE <campaign_name> {upper_campaign_name}"
         logger.error(msg)
@@ -399,8 +707,25 @@ def check_campaign_name(raw_dir, processed_dir):
     return upper_campaign_name
 
 
-def check_directories(raw_dir, processed_dir, force=False):
-    """Check that the specified directories respect the standards."""
+def check_directories(raw_dir: str, processed_dir: str, force: bool = False) -> tuple:
+    """Check that the specified directories respect the standards.
+
+    Parameters
+    ----------
+    raw_dir : str
+        Path of the raw directory
+    processed_dir : str
+        Path of the processed directory
+    force : bool, optional
+        If True, overwrite existing data into processed directory.
+        If False, raise an error if there are already data into processed directory.
+
+    Returns
+    -------
+    tuple
+        raw directory and processed directory
+    """
+
     raw_dir = parse_fpath(raw_dir)
     processed_dir = parse_fpath(processed_dir)
     check_raw_dir(raw_dir)
@@ -408,7 +733,20 @@ def check_directories(raw_dir, processed_dir, force=False):
     check_campaign_name(raw_dir, processed_dir)
     return raw_dir, processed_dir
 
-def check_metadata_dir(processed_path):
+
+def check_metadata_dir(processed_path: str) -> None:
+    """Check metadata folder
+
+    Parameters
+    ----------
+    processed_path : str
+        Path of the processed directory
+
+    Raises
+    ------
+    FileNotFoundError
+        Error metadat already existed or can not be created.
+    """
     # Create metadata folder
     try:
         metadata_folder_path = os.path.join(processed_path, "metadata")
@@ -422,9 +760,24 @@ def check_metadata_dir(processed_path):
         logger.exception(msg)
         raise FileNotFoundError(msg)
 
-def copy_metadata_from_raw_dir(raw_dir, processed_dir):
-    '''Copy yaml files in raw_dir/metadata into processed_dir/metadata'''
-    # Get src and dst metadata directory 
+
+def copy_metadata_from_raw_dir(raw_dir: str, processed_dir: str) -> None:
+    """Copy yaml files in raw_dir/metadata into processed_dir/metadata
+
+    Parameters
+    ----------
+    raw_dir : str
+        Path of the raw directory
+    processed_dir : str
+        Path of the processed directory
+
+    Raises
+    ------
+    ValueError
+        Error if the copy fails.
+    """
+
+    # Get src and dst metadata directory
     raw_metadata_dir = os.path.join(raw_dir, "metadata")
     processed_metadata_dir = os.path.join(processed_dir, "metadata")
     # Retrieve metadata fpaths in raw directory
@@ -434,35 +787,54 @@ def copy_metadata_from_raw_dir(raw_dir, processed_dir):
         # Check if is a files
         if os.path.isfile(raw_metadata_fpath):
             metadata_fname = os.path.basename(raw_metadata_fpath)
-            processed_metadata_fpath = os.path.join(processed_metadata_dir, metadata_fname)
+            processed_metadata_fpath = os.path.join(
+                processed_metadata_dir, metadata_fname
+            )
             try:
                 # Copy every file
                 shutil.copy(raw_metadata_fpath, processed_metadata_fpath)
-                msg = f'{metadata_fname} copied into {processed_metadata_dir}.' 
+                msg = f"{metadata_fname} copied into {processed_metadata_dir}."
                 logger.info(msg)
             except (Exception) as e:
-                msg = f'Something went wrong when copying {metadata_fname} into {processed_metadata_dir}.\n The error is: {e}.'
+                msg = f"Something went wrong when copying {metadata_fname} into {processed_metadata_dir}.\n The error is: {e}."
                 logger.error(msg)
         else:
-            msg = f'Cannot copy {metadata_fname} into {processed_metadata_dir}.'
+            msg = f"Cannot copy {metadata_fname} into {processed_metadata_dir}."
             logger.error(msg)
             raise ValueError(msg)
-    metadata_fnames = [os.path.basename(metadata_fpath) for metadata_fpath in raw_metadata_fpaths]
-    msg = f'The metadata of stations ({metadata_fnames}) have been copied into {processed_metadata_dir}.' 
+    metadata_fnames = [
+        os.path.basename(metadata_fpath) for metadata_fpath in raw_metadata_fpaths
+    ]
+    msg = f"The metadata of stations ({metadata_fnames}) have been copied into {processed_metadata_dir}."
     logger.info(msg)
-    
+
+
 ####--------------------------------------------------------------------------.
 #### L0 processing directory structure
 
 
-def create_directory_structure(raw_dir, processed_dir):
-    """Create directory structure for L0A and L0B processing."""
-    #-----------------------------------------------------.
+def create_directory_structure(raw_dir: str, processed_dir: str) -> None:
+    """Create directory structure for L0A and L0B processing.
+
+    Parameters
+    ----------
+    raw_dir : str
+        Path of the raw directory
+    processed_dir : str
+        Path of the processed directory
+
+    Raises
+    ------
+    FileNotFoundError
+        Error is folder structure can not be created.
+    """
+
+    # -----------------------------------------------------.
     #### Create metadata folder inside processed_dir
     check_metadata_dir(processed_dir)
     copy_metadata_from_raw_dir(raw_dir, processed_dir)
-    
-    #-----------------------------------------------------.
+
+    # -----------------------------------------------------.
     #### Create info folder inside processed_dir
     try:
         info_folder_path = os.path.join(processed_dir, "info")
@@ -475,8 +847,8 @@ def create_directory_structure(raw_dir, processed_dir):
         msg = f"Can not create folder metadata inside <info_folder_path>. Error: {e}"
         logger.exception(msg)
         raise FileNotFoundError(msg)
-        
-    #-----------------------------------------------------.
+
+    # -----------------------------------------------------.
     #### Create L0A folder inside processed_dir
     try:
         L0A_folder_path = os.path.join(processed_dir, "L0A")
@@ -489,8 +861,8 @@ def create_directory_structure(raw_dir, processed_dir):
         msg = f"Can not create folder L0A inside {processed_dir}. Error: {e}"
         logger.exception(msg)
         raise FileNotFoundError(msg)
-        
-    #-----------------------------------------------------.
+
+    # -----------------------------------------------------.
     #### Create L0B folder inside processed_dir
     try:
         L0B_folder_path = os.path.join(processed_dir, "L0B")
@@ -508,7 +880,9 @@ def create_directory_structure(raw_dir, processed_dir):
 
 ####--------------------------------------------------------------------------.
 #### DISDRODB L0A Readers
-def _read_L0A(fpath, lazy=True, verbose=False): 
+def _read_L0A(
+    fpath: str, lazy: bool = True, verbose: bool = False
+) -> Union[pd.DataFrame, dd.DataFrame]:
     # Log
     msg = f" - Reading L0 Apache Parquet file at {fpath} started."
     log_info(logger, msg, verbose)
@@ -520,75 +894,94 @@ def _read_L0A(fpath, lazy=True, verbose=False):
     # Log
     msg = f" - Reading L0 Apache Parquet file at {fpath} ended."
     log_info(logger, msg, verbose)
-    return df 
+    return df
 
-def read_L0A_dataframe(fpaths, lazy=True, verbose=False, debugging_mode=False):
-    """
-    Read DISDRODB L0A Apache Parquet file(s). 
+
+def read_L0A_dataframe(
+    fpaths: Union[str, list],
+    lazy: bool = True,
+    verbose: bool = False,
+    debugging_mode: bool = False,
+) -> Union[pd.DataFrame, dd.DataFrame]:
+    """Read DISDRODB L0A Apache Parquet file(s).
 
     Parameters
     ----------
     fpaths : str or list
         Either a list or a single filepath .
     verbose : bool
-        Whether to print detailed processing information into terminal. 
+        Whether to print detailed processing information into terminal.
         The default is False.
     debugging_mode : bool
         If True, it reduces the amount of data to process.
-        If fpaths is a list, it reads only the first 3 files 
+        If fpaths is a list, it reads only the first 3 files
         The default is False.
     lazy : bool
-        Whether to read the dataframe lazily with dask. 
+        Whether to read the dataframe lazily with dask.
         If lazy=True, it returns a dask.dataframe.
-        If lazy=False, it returns a pandas.DataFrame  
+        If lazy=False, it returns a pandas.DataFrame
         The default is True.
+
+    Returns
+    -------
+    Union[pd.DataFrame,dd.DataFrame]
+        Dataframe
+
+    Raises
+    ------
+    TypeError
+        Error if the reading fails
     """
+
     from disdrodb.L0.L0A_processing import concatenate_dataframe
+
     # ----------------------------------------
     # Check fpaths validity
-    if not isinstance(fpaths, (list,str)): 
+    if not isinstance(fpaths, (list, str)):
         raise TypeError("Expecting fpaths to be a string or a list of strings.")
     # TODO:
-    # - CHECK ENDS WITH .parquet  
+    # - CHECK ENDS WITH .parquet
     # ----------------------------------------
     # If list of length 1, convert to string
     if isinstance(fpaths, list) and len(fpaths) == 1:
         fpaths = fpaths[0]
     # ---------------------------------------------------
     # If more than 1 fpath, read and concantenate first
-    if isinstance(fpaths, list): 
-        if debugging_mode: 
-            fpaths = fpaths[0:3] # select first 3 fpaths
+    if isinstance(fpaths, list):
+        if debugging_mode:
+            fpaths = fpaths[0:3]  # select first 3 fpaths
         list_df = [_read_L0A(fpath, lazy=lazy, verbose=verbose) for fpath in fpaths]
         df = concatenate_dataframe(list_df, verbose=verbose, lazy=lazy)
-    # Else read the single file 
-    else: 
+    # Else read the single file
+    else:
         df = _read_L0A(fpaths, lazy=lazy, verbose=verbose)
     # ---------------------------------------------------
-    # Return dataframe 
-    return df 
+    # Return dataframe
+    return df
+
 
 ####---------------------------------------------------------------------------.
-#### TO BE DEPRECATED 
-#### Back-compatibility stuffs 
-# TODO: DEPRECATE 
-def get_L0_fname(campaign_name, station_id, suffix=""):
-    return get_L0A_fname(campaign_name, station_id, suffix)
 
 
-def get_L0_fpath(processed_dir, station_id, suffix=""):
-   return get_L0A_fpath(processed_dir, station_id, suffix)
+def check_L0_is_available(
+    processed_dir: str, station_id: str, suffix: str = ""
+) -> None:
+    """Check if the Apache parquet file has been found
 
+    Parameters
+    ----------
+    processed_dir : str
+        Path of the processed directory
+    station_id : int
+        Id of the station.
+    suffix : str, optional
+        Suffix, by default ""
 
-def get_L1_netcdf_fname(campaign_name, station_id, suffix=""):
-    return get_L0B_fname(campaign_name, station_id, suffix)
-
-
-def get_L1_netcdf_fpath(processed_dir, station_id, suffix=""):
-    return get_L0B_fpath(processed_dir, station_id, suffix)
-
-
-def check_L0_is_available(processed_dir, station_id, suffix=""):
+    Raises
+    ------
+    ValueError
+        Check if the Apache parquet file has not been found
+    """
     fpath = get_L0A_fpath(processed_dir, station_id, suffix=suffix)
     if not os.path.exists(fpath):
         msg = f"Need to run L0 processing first. The Apache Parquet file {fpath} is not available."
@@ -599,17 +992,46 @@ def check_L0_is_available(processed_dir, station_id, suffix=""):
     logger.info(msg)
 
 
-def read_L0_data(processed_dir, station_id, suffix="", 
-                 lazy=True, verbose=False, debugging_mode=False):
+def read_L0_data(
+    processed_dir: str,
+    station_id: str,
+    suffix: str = "",
+    lazy: bool = True,
+    verbose: bool = False,
+    debugging_mode: bool = False,
+) -> Union[pd.DataFrame, dd.DataFrame]:
     """Read L0 Apache Parquet into dataframe.
-    
-    If debugging_mode = True, return just a subset of total rows.
+
+
+    Parameters
+    ----------
+    processed_dir : str
+        Path of the processed directory
+    station_id : int
+        Id of the station.
+    suffix : str, optional
+        Suffix, by default ""
+    lazy : bool, optional
+        If True : Dask is used.
+        If False : Pandas is used
+        by default True
+    verbose : bool, optional
+        Wheter to verbose the processing.
+        The default is False.
+    debugging_mode : bool, optional
+        If True, return just a subset of total rows, by default False
+
+    Returns
+    -------
+    Union[pd.DataFrame,dd.DataFrame]
+        Dataframe
     """
+
     # Check L0 is available
     check_L0_is_available(processed_dir, station_id, suffix=suffix)
     # Define fpath
     fpath = get_L0A_fpath(processed_dir, station_id, suffix=suffix)
-    # Read L0A Apache Parquet file 
+    # Read L0A Apache Parquet file
     df = _read_L0A(fpath, lazy=lazy, verbose=verbose)
     # Subset dataframe if debugging_mode = True
     if debugging_mode:
@@ -621,4 +1043,3 @@ def read_L0_data(processed_dir, station_id, suffix="",
 
 
 ####--------------------------------------------------------------------------.
-
